defmodule Chronoscope.NTS.Behaviour do
  @callback(key_establishment(host :: String.t(), port :: integer()) :: {:ok, Map.t()}, {:error, any()})
end

defmodule Chronoscope.NTS do
  @behaviour Chronoscope.NTS.Behaviour

  require Logger

  alias Chronoscope.NTS

  def healthy?() do
    true
  end

  def list() do
    NTS.DynamicSupervisor
    |> DynamicSupervisor.which_children()
    |> Enum.map(fn {_, pid, _, _} -> GenServer.call(pid, :list) end)
  end

  @impl true
  def key_establishment(host, port) do
    name = "#{host}:#{port}"

    case Registry.lookup(NTS.Registry, name) do
      [{pid, _}] ->
        GenServer.call(pid, :key_establishment)

      [] ->
        {:ok, pid} =
          DynamicSupervisor.start_child(
            NTS.DynamicSupervisor,
            {NTS.Client, host: host, port: port, name: {:via, Registry, {NTS.Registry, name}}}
          )

        GenServer.call(pid, :key_establishment)
    end
  end
end
